﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.UI.Xaml;

namespace CSHTML5.Extensions
{
    public static class DomIdentifierExtensions
    {
        public static readonly DependencyProperty DomIdProperty = DependencyProperty.RegisterAttached(@"DomId",
                                                                                                      typeof(string),
                                                                                                      typeof(FrameworkElement),
                                                                                                      new PropertyMetadata(null, DomIdProperty_Changed));

        private static async void DomIdProperty_Changed(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            FrameworkElement frameworkElement = (FrameworkElement)d;

            string id = (e.NewValue ?? "").ToString();

            await WaitForControlToBeInVisualTree(frameworkElement);

            var div = CSHTML5.Interop.GetDiv(frameworkElement);

            if (!CSHTML5.Interop.IsRunningInTheSimulator)
            {
                CSHTML5.Interop.ExecuteJavaScript("$0.id = $1", div, id);
            }
            else
            {
                // We cannot set the ID in the Simulator because it uses IDs for internal purposes, to relate the DOM elements to their XAML counterparts.
            }
        }

        static Task WaitForControlToBeInVisualTree(FrameworkElement frameworkElement)
        {
            var taskCompletionSource = new TaskCompletionSource<object>();
            if (frameworkElement.IsLoaded)
            {
                taskCompletionSource.SetResult(null);
            }
            else
            {
                frameworkElement.Loaded += (s, e) =>
                {
                    //todo: ideally we should unregister the Loaded event here.
                    taskCompletionSource.TrySetResult(null);
                };
            }
            return taskCompletionSource.Task;
        }

        public static string GetDomId(this FrameworkElement frameworkElement)
        {
            if (null == frameworkElement)
            {
                throw new ArgumentNullException(nameof(frameworkElement));
            }

            string value = (string)frameworkElement.GetValue(DomIdProperty);
            return value;
        }

        public static void SetDomId(this FrameworkElement frameworkElement, string value)
        {
            if (null == frameworkElement)
            {
                throw new ArgumentNullException(nameof(frameworkElement));
            }

            frameworkElement.SetValue(DomIdProperty, value);
        }
    }
}
